Package: clustSIGNAL
Type: Package
Title: ClustSIGNAL: a spatial clustering method
Version: 1.4.0
Authors@R: c(
    person("Pratibha", "Panwar", email = "pratibhapanwar.4@gmail.com", 
    role = c("cre", "aut", "ctb"), comment = c(ORCID = "0000-0002-7437-7084")),
    person("Boyi", "Guo", email = "boyi.guo.work@gmail.com", role = "aut"),
    person("Haowen", "Zhao", email = "haowen.zhou19@gmail.com", role = "aut"),
    person("Stephanie", "Hicks", email = "shicks19@jhu.edu", role = "aut"),
    person("Shila", "Ghazanfar", email = "shazanfar@gmail.com", 
    role = c("aut", "ctb"), comment = c(ORCID = "0000-0001-7861-6997")))
Description: clustSIGNAL: clustering of Spatially Informed Gene expression with 
    Neighbourhood Adapted Learning. A tool for adaptively smoothing and clustering 
    gene expression data. clustSIGNAL uses entropy to measure heterogeneity of 
    cell neighbourhoods and performs a weighted, adaptive smoothing, where 
    homogeneous neighbourhoods are smoothed more and heterogeneous neighbourhoods 
    are smoothed less. This not only overcomes data sparsity but also incorporates 
    spatial context into the gene expression data. The resulting smoothed gene 
    expression data is used for clustering and could be used for other downstream 
    analyses.
License: GPL-2
Encoding: UTF-8
URL: https://sydneybiox.github.io/clustSIGNAL/
BugReports: https://github.com/sydneybiox/clustSIGNAL/issues
biocViews: Clustering, Software, GeneExpression, Spatial,
        Transcriptomics, SingleCell
RoxygenNote: 7.3.2
Depends: R (>= 4.4.0), SpatialExperiment
Imports: BiocParallel, BiocNeighbors, bluster (>= 1.16.0), scater,
        harmony, SingleCellExperiment, SummarizedExperiment, methods,
        Matrix, reshape2
Suggests: knitr, BiocStyle, testthat (>= 3.0.0), aricode, ggplot2,
        patchwork, dplyr, scattermore
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/clustSIGNAL
git_branch: RELEASE_3_23
git_last_commit: 420ae66
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:18:47 UTC; biocbuild
Author: Pratibha Panwar [cre, aut, ctb] (ORCID:
    <https://orcid.org/0000-0002-7437-7084>),
  Boyi Guo [aut],
  Haowen Zhao [aut],
  Stephanie Hicks [aut],
  Shila Ghazanfar [aut, ctb] (ORCID:
    <https://orcid.org/0000-0001-7861-6997>)
Maintainer: Pratibha Panwar <pratibhapanwar.4@gmail.com>
