# Generated by roxygen2: do not edit by hand

export(adaptiveSmoothing)
export(clustSIGNAL)
export(entropyMeasure)
export(neighbourDetect)
export(p1_clustering)
export(p2_clustering)
importFrom(BiocNeighbors,KmknnParam)
importFrom(BiocNeighbors,findKNN)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,SnowParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpparam)
importFrom(Matrix,sparseMatrix)
importFrom(SingleCellExperiment,logcounts)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SpatialExperiment,spatialCoords)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(bluster,KmeansParam)
importFrom(bluster,NNGraphParam)
importFrom(bluster,TwoStepParam)
importFrom(bluster,clusterRows)
importFrom(harmony,RunHarmony)
importFrom(methods,as)
importFrom(methods,show)
importFrom(reshape2,melt)
importFrom(scater,runPCA)
importFrom(stats,dexp)
importFrom(stats,dnorm)
importFrom(stats,setNames)
