% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{.generateBPParam}
\alias{.generateBPParam}
\title{Generating BPParam object}
\usage{
.generateBPParam(cores = 1)
}
\arguments{
\item{cores}{Desired number of cores for BPPARAM object.}
}
\value{
A BPPPARAM object.
}
\description{
A utility function to generate BPPARAM object.
}
\keyword{internal}
